import {Injectable} from '@angular/core'
import {ApiService} from '../../shared/service/api.service'
import {Observable} from 'rxjs'
import {Recipe, RecipeStep} from '../../shared/model/recipe.model'
import {map} from 'rxjs/operators'

@Injectable({
  providedIn: 'root'
})
export class RecipeService {
  constructor(
    private api: ApiService
  ) {
  }

  get all(): Observable<Recipe[]> {
    return this.api.get<Recipe[]>('/recipe')
  }

  getAllByName(name: string): Observable<Recipe[]> {
    return this.api.get<Recipe[]>(`/recipe?name=${name}`)
  }

  get allSortedByCompany(): Observable<{ company: string, recipes: Recipe[] }[]> {
    return this.all.pipe(map(recipes => {
      const mapped = []
      recipes.forEach(r => {
        if (!mapped[r.company.id]) {
          mapped[r.company.id] = {company: r.company.name, recipes: []}
        }
        mapped[r.company.id].recipes.push(r)
      })
      return mapped.filter(e => e != null)  // Filter to remove empty elements in the array that appears for some reason
    }))
  }

  getById(id: number): Observable<Recipe> {
    return this.api.get<Recipe>(`/recipe/${id}`)
  }

  save(name: string, description: string, color: string, gloss: number, sample: number, approbationDate: string, remark: string, companyId: number): Observable<Recipe> {
    const body = {name, description, color, gloss, sample, remark, companyId}
    if (approbationDate) {
      // @ts-ignore
      body.approbationDate = approbationDate
    }
    return this.api.post<Recipe>('/recipe', body)
  }

  update(id: number, name: string, description: string, color: string, gloss: number, sample: number, approbationDate: string, remark: string, steps: Map<number, RecipeStep[]>) {
    const body = {id, name, description, color, gloss, sample, remark, steps: []}
    if (approbationDate) {
      // @ts-ignore
      body.approbationDate = approbationDate
    }

    steps.forEach((groupSteps, groupId) => {
      const mappedGroupSteps = groupSteps.map(s => {
        return {message: s.message, position: s.position}
      })
      body.steps.push({groupId, steps: mappedGroupSteps})
    })

    return this.api.put<Recipe>('/recipe', body)
  }

  updateExplorerModifications(id: number, notes: Map<number, string>, mixesLocationChange: Map<number, string>): Observable<void> {
    const body = {
      recipeId: id,
      notes: [],
      mixesLocation: []
    }

    notes.forEach((content, groupId) => {
      body.notes.push({groupId, content})
    })

    mixesLocationChange.forEach((location, mixId) => {
      body.mixesLocation.push({mixId, location})
    })

    return this.api.put<void>('/recipe/public', body)
  }

  delete(id: number): Observable<void> {
    return this.api.delete<void>(`/recipe/${id}`)
  }
}
