import {Component, ViewChild} from '@angular/core'
import {FormField} from '../../../shared/components/entity-add/entity-add.component'
import {Validators} from '@angular/forms'
import {map} from 'rxjs/operators'
import {MaterialTypeService} from '../../../material-type/service/material-type.service'
import {MaterialService} from '../../service/material.service'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {Material, openSimdut} from '../../../shared/model/material.model'
import {ErrorHandler, ErrorService} from '../../../shared/service/error.service'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-edit',
  templateUrl: './edit.component.html',
  styleUrls: ['./edit.component.sass']
})
export class EditComponent extends ErrorHandlingComponent {
  @ViewChild('simdutTemplate', {static: true}) simdutTemplateRef

  material: Material | null
  formFields: FormField[] = [
    {
      name: 'name',
      label: 'Code',
      icon: 'form-textbox',
      type: 'text',
      required: true,
      errorMessages: [
        {conditionFn: (errors) => errors.required, message: 'Un code est requis'}
      ]
    },
    {
      name: 'inventoryQuantity',
      label: 'Quantité en inventaire',
      icon: 'beaker-outline',
      type: 'number',
      required: true,
      validator: Validators.min(0),
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Une quantité en inventaire est requise'},
        {conditionFn: errors => errors.min, message: 'La quantité doit être supérieure ou égale à 0'}
      ],
      step: '0.01'
    },
    {
      name: 'materialType',
      label: 'Type de produit',
      icon: 'shape-outline',
      type: 'select',
      required: true,
      errorMessages: [
        {conditionFn: errors => errors.required, message: 'Un type de produit est requis'}
      ],
      valueFn: material => material.materialType.id,
      options$: this.materialTypeService.all.pipe(map(types => types.map(t => {
        return {value: t.id, label: t.name}
      })))
    },
    {
      name: 'simdutFile',
      label: 'Fiche signalitique',
      icon: 'file-outline',
      type: 'file',
      fileType: 'application/pdf'
    }
  ]
  selectedSimdutFile: File | null

  errorHandlers: ErrorHandler[] = [{
    filter: error => error.type === 'notfound-material-id',
    consumer: error => this.urlUtils.navigateTo('/catalog/material/list')
  }, {
    filter: error => error.type === 'exists-material-name',
    messageProducer: error => `Un produit avec le nom '${error.name}' existe déjà`
  }, {
    filter: error => error.type === 'cannotdelete-material',
    messageProducer: error => `Ce produit est utilisé dans une plusieurs recettes`
  }]

  constructor(
    private materialService: MaterialService,
    private materialTypeService: MaterialTypeService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    this.formFields[3].template = this.simdutTemplateRef

    const id = parseInt(this.activatedRoute.snapshot.paramMap.get('id'))
    this.subscribeEntityById(
      this.materialService,
      id,
      material => {
        this.appState.title = `${material.name} (Modifications)`
        this.material = material
      }
    )
  }

  submit(values) {
    this.subscribeAndNavigate(
      this.materialService.update(this.material.id, values.name, values.inventoryQuantity, values.materialType, this.selectedSimdutFile),
      '/catalog/material/list'
    )
  }

  delete() {
    this.subscribeAndNavigate(
      this.materialService.delete(this.material.id),
      '/catalog/material/list'
    )
  }

  get hasSimdut(): boolean {
    return this.material.simdutUrl != null
  }

  openSimdut() {
    openSimdut(this.material)
  }
}
