import {Injectable} from '@angular/core'
import {AlertService} from './alert.service'
import {AppState} from '../app-state'

@Injectable({
  providedIn: 'root'
})
export class ErrorService {
  private static readonly UNKNOWN_ERROR_MESSAGE = 'Une erreur inconnue est survenue'
  private static readonly DEFAULT_ERROR_HANDLERS: ErrorHandler[] = [{
    filter: error => error.type.startsWith('notfound'),
    messageProducer: () => 'Une ressource demandée n\'a pas été trouvée'
  }, {
    filter: error => error.type.startsWith('exists'),
    messageProducer: () => 'Un conflit avec une autre ressource s\'est produit'
  }, {
    filter: error => error.type.startsWith('cannotdelete'),
    messageProducer: () => 'Une ressource n\'a pas pu être supprimée'
  }, {
    filter: error => error.status === 400,
    messageProducer: () => 'Les données envoyées n\'étaient pas valides'
  }, {
    filter: error => error.status === 401,
    messageProducer: () => 'Vous devez être connecté pour effectuer cette action'
  }, {
    filter: error => error.status === 403,
    messageProducer: () => 'Vous n\'avez pas la permission d\'effectuer cette action'
  }, {
    filter: error => error.status === 500,
    messageProducer: () => ErrorService.UNKNOWN_ERROR_MESSAGE
  }]

  private activeHandler: ErrorHandlerComponent

  constructor(
    private alertService: AlertService,
    private appState: AppState
  ) {
  }

  handleError(response: any) {
    let matchingModels

    if (isServerOfflineError(response)) {
      this.appState.isServerOnline = false
      return
    }

    const error = response.error
    if (!error || !error.type) {
      return
    }

    if (this.activeHandler) {
      matchingModels = this.activeHandler.errorHandlers.filter(m => m.filter(error)) // Find error models whose filter matches the current error
    } else {
      console.warn('An error occurred but no handler was set')
      console.error(error)
    }

    if (!matchingModels || matchingModels.length == 0) {  // If none are found, search in defaults handlers
      matchingModels = ErrorService.DEFAULT_ERROR_HANDLERS.filter(m => m.filter(error))
      console.error(error)
    }

    if (!matchingModels || matchingModels.length == 0) { // If still none are found, handle as an unknown error
      this.consumeUnknownError(error)
      return
    }

    matchingModels.forEach(m => {
      if (m.consumer || m.messageProducer) {
        if (m.consumer) {
          m.consumer(error)
        }
        if (m.messageProducer) {
          this.alertService.pushError(m.messageProducer(error))
        }
      } else {
        console.error('An error model has no consumer or message')
      }
    })
  }

  consumeUnknownError(error: any) {
    console.error('An unknown error occurred', error)
    this.pushUnknownError()
  }

  pushUnknownError() {
    this.alertService.pushError(ErrorService.UNKNOWN_ERROR_MESSAGE)
  }

  set activeErrorHandler(handler: ErrorHandlerComponent) {
    this.activeHandler = handler
  }
}

/**
 * An error handler, defining models of errors that this type will handle
 */
export interface ErrorHandlerComponent {
  errorHandlers: ErrorHandler[]
}

/**
 * An error model define how errors matching its filter will be handled.
 *
 * The consumer will consume matching errors when they occurs.
 * The message producer returns a string that will be pushed to the alert system.
 *
 * To work correctly a model must define at least one handler (consumer or message producer).
 */
export class ErrorHandler {
  constructor(
    public filter: (error: any) => Boolean,
    public consumer?: (error: any) => void,
    public messageProducer?: (error: any) => String
  ) {
  }
}

function isServerOfflineError(response: any): boolean {
  return response.status === 0 || response.status === 502
}
