import {Component, ViewChild} from '@angular/core'
import {Validators} from '@angular/forms'
import {currentPermissionsFieldComponent} from '../../../shared/components/permissions-field/permissions-field.component'
import {GroupService} from '../../../groups/services/group.service'
import {UserService} from '../../services/user.service'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {ErrorHandler, ErrorService} from '../../../shared/service/error.service'
import {FormField} from '../../../shared/components/entity-add/entity-add.component'
import {map} from 'rxjs/operators'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-add',
  templateUrl: './add.component.html',
  styleUrls: ['./add.component.sass']
})
export class AddComponent extends ErrorHandlingComponent {
  @ViewChild('permissionsTemplateRef', {static: true}) permissionsTemplateRef

  formFields: FormField[] = [{
    name: 'id',
    label: 'Numéro d\'utilisateur',
    icon: 'pound',
    type: 'number',
    required: true,
    validator: Validators.compose([Validators.pattern(new RegExp('^[0-9]+$')), Validators.min(0)]),
    errorMessages: [
      {conditionFn: errors => errors.required, message: 'Un numéro d\'utilisateur est requis'},
      {conditionFn: errors => errors.pattern, message: 'Le numéro d\'utilisateur doit être un nombre'}
    ]
  }, {
    name: 'firstName',
    label: 'Prénom',
    icon: 'account',
    type: 'text',
    required: true,
    errorMessages: [
      {conditionFn: errors => errors.required, message: 'Un prénom est requis'}
    ]
  }, {
    name: 'lastName',
    label: 'Nom',
    icon: 'account',
    type: 'text',
    required: true,
    errorMessages: [
      {conditionFn: errors => errors.required, message: 'Un nom est requis'}
    ]
  }, {
    name: 'password',
    label: 'Mot de passe',
    icon: 'lock',
    type: 'password',
    required: true,
    validator: Validators.minLength(8),
    errorMessages: [
      {conditionFn: errors => errors.required, message: 'Un mot de passe est requis'},
      {conditionFn: errors => errors.minlength, message: 'Le mot de passe doit comprendre au moins 8 caractères'}
    ]
  }, {
    name: 'groupId',
    label: 'Groupe',
    icon: 'account-multiple',
    type: 'select',
    defaultValue: -1,
    options$: this.groupService.allWithDefault.pipe(map(groups => groups.map(g => {
      return {value: g.id, label: g.name}
    })))
  }, {
    name: 'permissions',
    label: 'Permissions',
    type: 'permissions'
  }]

  errorHandlers: ErrorHandler[] = [{
    filter: error => error.type === 'exists-user-id',
    messageProducer: error => `Un utilisateur avec l'identifiant ${error.id} existe déjà`
  }, {
    filter: error => error.type === 'exists-user-fullName',
    messageProducer: error => `Un utilisateur nommé '${error.fullName}' existe déjà`
  }]

  constructor(
    private userService: UserService,
    private groupService: GroupService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Nouvel utilisateur'
  }

  ngOnInit() {
    super.ngOnInit()

    this.formFields[this.formFields.length - 1].template = this.permissionsTemplateRef
  }

  submit(values) {
    const permissionsField = currentPermissionsFieldComponent
    if (permissionsField.valid()) {
      this.subscribeAndNavigate(
        this.userService.save(
          values.id,
          values.firstName,
          values.lastName,
          values.password,
          values.groupId,
          permissionsField.allEnabledPermissions
        ),
        '/admin/user/list'
      )
    }
  }
}
